@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005 Christophe Fergeau
+ *  Copyright (C) 2005-2007 Christophe Fergeau
  *
  * 
  *  The code contained in this file is free software; you can redistribute
@@ -330,7 +330,7 @@ write_mhod_type_1 (gchar *string, iPodBuffer *buffer)
 
 	padding = 4 - ( (total_bytes + len) % 4 );
 	mhod->padding = padding;
-	mhod->type = get_gint32 (0x0001, buffer->byte_order);
+	mhod->type = get_gint16 (0x01, buffer->byte_order);
 
 	/* Make sure we have enough free space to write the string */
 	if (ipod_buffer_maybe_grow (buffer, total_bytes + len + padding ) != 0) {
@@ -384,7 +384,7 @@ write_mhod_type_3 (gchar *string, iPodBuffer *buffer)
 	    /* number of bytes of the string encoded in UTF-16 */
 	    mhod->string_len = get_gint32 (2*len, buffer->byte_order);
 	    padding = 4 - ( (total_bytes + 2*len) % 4 );
-	    mhod->padding = padding; /* high byte is padding length (0-3) */
+	    mhod->padding = padding;
 
 	    /* Make sure we have enough free space to write the string */
 	    if (ipod_buffer_maybe_grow (buffer, total_bytes + 2*len+padding) != 0) {
@@ -410,7 +410,7 @@ write_mhod_type_3 (gchar *string, iPodBuffer *buffer)
 	    /* e.g. len = 7 bytes, len%4 = 3, 4-3=1 -> requires 1 byte
 	       padding */
 	    padding = 4 - ( (total_bytes + len) % 4 );
-	    mhod->padding = padding; /* high byte is padding length (0-3) */
+	    mhod->padding = padding;
 	    /* Make sure we have enough free space to write the string */
 	    if (ipod_buffer_maybe_grow (buffer, total_bytes + 2*len+padding) != 0) {
 		return  -1;
@@ -531,6 +531,8 @@ write_mhii (Itdb_DB *db, void *data, iPodBuffer *buffer)
 	GList *it = NULL;
 	Itdb_Track *song;
 	Itdb_Artwork *artwork;
+	guint64 mactime;
+	Itdb_Device *device = db_get_device (db);
 
 	mhii = (MhiiHeader *)init_header (buffer, "mhii", sizeof (MhiiHeader));
 	if (mhii == NULL) {
@@ -555,11 +557,14 @@ write_mhii (Itdb_DB *db, void *data, iPodBuffer *buffer)
 	mhii->unknown4 = get_gint32 (artwork->unk028, buffer->byte_order);
 	mhii->rating = get_gint32 (artwork->rating, buffer->byte_order);
 	mhii->unknown6 = get_gint32 (artwork->unk036, buffer->byte_order);
-	mhii->orig_date = get_guint32 (artwork->creation_date, buffer->byte_order);
-	mhii->digitized_date = get_guint32 (artwork->digitized_date,
-					    buffer->byte_order);
-	mhii->orig_img_size = get_gint32 (artwork->artwork_size, 
-					  buffer->byte_order);
+
+	mactime = device_time_time_t_to_mac (device, artwork->creation_date);
+	mhii->orig_date = get_guint32 (mactime, buffer->byte_order);
+
+	mactime = device_time_time_t_to_mac (device, artwork->digitized_date);
+	mhii->digitized_date = get_guint32 (mactime, buffer->byte_order);
+
+	mhii->orig_img_size = get_gint32 (artwork->artwork_size, buffer->byte_order);
 	it = artwork->thumbnails;
 	num_children = 0;
 	while (it != NULL) {
@@ -957,7 +962,7 @@ write_mhfd (Itdb_DB *db, iPodBuffer *buffer, int id_max)
 		mhfd->unknown2 = get_gint32 (2, buffer->byte_order);
 		break;
 	case DB_TYPE_ITUNES:
-		mhfd->unknown2 = get_gint32 (1, buffer->byte_order);
+		mhfd->unknown2 = get_gint32 (2, buffer->byte_order);
 		break;
 	}
 	mhfd->next_id = get_gint32 (id_max, buffer->byte_order);
