@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005 Christophe Fergeau
+ *  Copyright (C) 2005-2007 Christophe Fergeau
  *
  * 
  *  The code contained in this file is free software; you can redistribute
@@ -20,6 +20,7 @@
  * 
  *  This product is not supported/written/published by Apple!
  *
+ * $Id: db-itunes-parser.h 1514 2007-06-01 13:56:32Z jcsjcs $
  */
 
 
@@ -49,13 +50,15 @@ typedef struct _MhlHeader MhlHeader;
 
 
 typedef struct _MhbdHeader MhbdHeader;
-typedef struct _MhsdHeader MhsdHeader;
+typedef struct _ArtworkDB_MhsdHeader ArtworkDB_MhsdHeader;
+/*typedef struct _MhsdHeader MhsdHeader;*/
 typedef struct _MhltHeader MhltHeader;
 typedef struct _MhlpHeader MhlpHeader;
 typedef struct _MhypHeader MhypHeader;
 typedef struct _MhipHeader MhipHeader;
 typedef struct _MhitHeader MhitHeader;
-typedef struct _MhodHeader MhodHeader;
+typedef struct _ArtworkDB_MhodHeader ArtworkDB_MhodHeader;
+/*typedef struct _MhodHeader MhodHeader;*/
 typedef struct _MhfdHeader MhfdHeader;
 typedef struct _MhliHeader MhliHeader;
 typedef struct _MhiiHeader MhiiHeader;
@@ -68,9 +71,10 @@ typedef struct _MhiaHeader MhiaHeader;
 
 typedef struct _MhitHeader471 MhitHeader471;
 /* MHOD typedef mess */
+typedef struct _ArtworkDB_MhodHeaderArtworkType3 ArtworkDB_MhodHeaderArtworkType3;
 typedef struct _MhodHeaderString MhodHeaderString;
 typedef struct _MhodHeaderArtworkType1 MhodHeaderArtworkType1;
-typedef struct _MhodHeaderArtworkType3 MhodHeaderArtworkType3;
+/* typedef struct _MhodHeaderArtworkType3 MhodHeaderArtworkType3; */
 typedef struct _MhodHeaderSmartPlaylistData MhodHeaderSmartPlaylistData;
 typedef struct _MhodHeaderSmartPlaylistRuleString MhodHeaderSmartPlaylistRuleString; 
 typedef struct _MhodHeaderSmartPlaylistRuleNonString MhodHeaderSmartPlaylistRuleNonString;
@@ -99,14 +103,27 @@ enum MhsdPhotoIndexType {
 	MHSD_FILE_LIST = 3
 };
 
-struct _MhsdHeader {
+struct _ArtworkDB_MhsdHeader {
+	unsigned char header_id[4];
+	gint32 header_len;
+	gint32 total_len;
+        /* Strangely, the following field is only 16 bits long in the
+	 * ArtworkDB (it's definitely 32 bits in the iTunesDB). This
+	 * could well be an error with the first generation of mobile
+	 * phones with iPod support).
+	 */
+	gint16 index;
+        gint16 unknown014;
+	unsigned char padding[];
+};
+
+struct _iTunesDB_MhsdHeader {
 	unsigned char header_id[4];
 	gint32 header_len;
 	gint32 total_len;
 	gint32 index;
 	unsigned char padding[];
 };
-
 struct _MhltHeader {
 	unsigned char header_id[4];
 	gint32 header_len;
@@ -305,6 +322,21 @@ struct _MhodHeader {
 	gint32 unknown2;
 };
 
+struct _ArtworkDB_MhodHeader {
+	unsigned char header_id[4];
+	gint32 header_len;
+	gint32 total_len;
+    /* Strangely, the following field is only 16 bits long in the
+	 * ArtworkDB (it's definitely 32 bits in the iTunesDB). This
+	 * could well be an error with the first generation of mobile
+	 * phones with iPod support).
+	 */
+	gint16 type;
+        gint16 unknown014;
+	gint32 unknown1;
+	gint32 unknown2;
+};
+
 struct _MhodHeaderString {
 	unsigned char header_id[4];
 	gint32 header_len;
@@ -334,7 +366,9 @@ struct _MhodHeaderArtworkType1 {
 	unsigned char header_id[4];
 	gint32 header_len;
 	gint32 total_len;
-	gint32 type; /* low 3 bytes are type (always 1); high byte is padding length (0-3) */
+	gint16 type; 
+	gint8 unknown;
+	gint8 padding;
 	gint32 unknown1;
 	gint32 unknown2;
 	gint32 string_len;
@@ -343,11 +377,14 @@ struct _MhodHeaderArtworkType1 {
 	unsigned char string[];
 };
 
+#if 0
+/* not used at all */
 struct _MhodHeaderArtworkType3 {
 	unsigned char header_id[4];
 	gint32 header_len;
 	gint32 total_len;
-	gint32 type; /* 3 */
+	gint16 type; /* 3 */
+	gint16 padding; /* high byte is padding length (0-3) */
 	gint32 unknown1;
 	gint32 unknown2;
 	gint32 string_len;
@@ -355,6 +392,26 @@ struct _MhodHeaderArtworkType3 {
 	gint32 unknown4;
 	gunichar2 string[];
 };
+#endif
+
+struct _ArtworkDB_MhodHeaderArtworkType3 {
+	unsigned char header_id[4];
+	gint32 header_len;
+	gint32 total_len;
+        gint16 type; /* 3 */
+        gint8  unknown13;
+	gint8  padding;
+	gint32 unknown1;
+	gint32 unknown2;
+	gint32 string_len;
+	gint8 mhod_version; /* 0,1: string is UTF8, 2: string is
+			       UTF16-LE */
+        gint8 unknown5;
+        gint16 unknown6;
+	gint32 unknown4;
+        gchar  string[];
+};
+
 
 enum MhodLimitType {
 	MHOD_LIMIT_MINUTES = 1,
@@ -488,13 +545,19 @@ struct _MhfdHeader {
 	gint32 header_len;
 	gint32 total_len;
 	gint32 unknown1;
-	gint32 unknown2;
+	gint32 unknown2; /* always seem to be 1 for older databases,
+			    in an ArtworkDB generated by iTunes 4.9 or
+			    above, it's 2. Caution: iTunes7 removes
+			    the whole database if this field is 1 */
 	gint32 num_children;
 	gint32 unknown3;
-	gint32 unknown4;
+	gint32 next_id;
 	gint64 unknown5;
 	gint64 unknown6;
-	gint32 unknown7;
+	gint8  unknown_flag1;
+	gint8  unknown_flag2;
+	gint8  unknown_flag3;
+	gint8  unknown_flag4;
 	gint32 unknown8;
 	gint32 unknown9;
 	gint32 unknown10;
@@ -517,10 +580,10 @@ struct _MhiiHeader {
 	gint32 image_id;
 	gint64 song_id;
 	gint32 unknown4;
-	gint32 unknown5;
+	gint32 rating;
 	gint32 unknown6;
-	gint32 unknown7;
-	gint32 unknown8;
+	gint32 orig_date;
+	gint32 digitized_date;
 	gint32 orig_img_size;
 	unsigned char padding[];
 } __attribute__((__packed__));
@@ -538,7 +601,8 @@ struct _MhniHeader {
 	gint32 correlation_id;
 	gint32 ithmb_offset;
 	gint32 image_size;
-	gint32 unknown3;
+	gint16 vertical_padding;
+	gint16 horizontal_padding;
 	gint16 image_height;
 	gint16 image_width;
 	unsigned char padding[];
@@ -555,14 +619,29 @@ struct _MhbaHeader {
 	unsigned char header_id[4];
 	gint32 header_len;
 	gint32 total_len;
-	gint32 num_mhods; /* number of Data Objects in the List, probably always 1 */
-	gint32 num_mhias; /* number of pictures in the album */
-	gint32 playlist_id; /* starts out at $64 and increments by 1 */
-	gint32 unknown2; /* unknown, seems to be always 0 */
-	gint32 unknown3; /* unknown, but is 0x10000 for the Photo Library and 0x60000 for normal albums
-			  * (maybe not a 4 byte field?) */
-	gint32 unknown[7]; /* more zeroes */
-	gint32 prev_playlist_id; /* the id of the previous playlist */
+	gint32 num_mhods;  /* number of Data Objects in the List,
+			      probably always 1 */
+	gint32 num_mhias;  /* number of pictures in the album */
+	gint32 album_id;   /* starts out at $64 and increments by 1 */
+	gint32 unk024;     /* unknown, seems to be always 0 */
+	gint16 unk028;     /* unknown, seems to be always 0 */
+	guint8 album_type; /* 1 = master photo list ("Photo Library"),
+			      2 = normal album, sometimes 4 and 5 */
+	guint8 playmusic;  /* play music during slideshow (from iPhoto setting) */
+	guint8 repeat;     /* repeat the slideshow (from iPhoto setting) */
+	guint8 random;     /* show the slides in random order (from iPhoto
+			      setting) */
+        guint8 show_titles;/* show slide captions (from iPhoto setting) */
+	guint8 transition_direction; /* 0=none, 1=left-to-right,
+			      2=right-to-left, 3=top-to-bottom,
+			      4=bottom-to-top (from iPhoto setting) */
+        gint32 slide_duration; /* in seconds (from iPhoto setting) */
+	gint32 transition_duration; /* in milliseconds (from iPhoto setting) */
+        gint32 unk044;     /* unknown, seems to always be 0 */
+	gint32 unk048;     /* unknown, seems to always be 0 */
+	gint64 song_id;    /* dbid2 of track in iTunesDB to play during
+			      slideshow (from iPhoto setting) */
+	gint32 prev_album_id; /* the id of the previous photoalbum */
 	unsigned char padding[];
 };
 
@@ -589,7 +668,7 @@ struct _MhiaHeader {
 	gint32 total_len; /* probably the size of the header and all child records;
 			   * as there aren't any child records this is equal to header length */
 	gint32 unknown1; /* seems to be zero */
-	gint32 image_id; /* the id of the mhii record this mhia refers to */
+	guint32 image_id; /* the id of the mhii record this mhia refers to */
 	unsigned char padding[];
 };
 
